@@ -15,15 +15,17 @@ Versioning semantics:
 
 __intname__ = "ofunctions.file_utils"
 __author__ = "Orsiris de Jong"
-__copyright__ = "Copyright (C) 2017-2021 Orsiris de Jong"
+__copyright__ = "Copyright (C) 2017-2022 Orsiris de Jong"
 __description__ = "File/dir/permissions/time handling"
 __licence__ = "BSD 3 Clause"
-__version__ = "1.0.2"
-__build__ = "2021100601"
+__version__ = "1.2.4"
+__build__ = "2022110601"
+__compat__ = "python2.7+"
 
 import json
 import logging
 import os
+import sys
 import re
 import shutil
 from ofunctions import random
@@ -32,11 +34,38 @@ from datetime import datetime
 from fnmatch import fnmatch
 from itertools import chain
 from threading import Lock
-from typing import Callable, Iterable, Union, Optional
+
+# Python 2.7 compat fixes
+try:
+    from typing import Callable, Iterable, Union, Optional
+except ImportError:
+    pass
+if sys.version_info[0] < 3:
+
+    class FileNotFoundError(OSError):
+        pass
+
+    class PermissionError(OSError):
+        pass
+
+
+# Python 2.7 compat where datetime.now() does not have .timestamp() method
+if sys.version_info[0] < 3 or sys.version_info[1] < 4:
+    # python version < 3.3
+    import time
+
+    def get_timestamp(date):
+        return time.mktime(date.timetuple())
+
+else:
+
+    def get_timestamp(date):
+        return date.timestamp()
+
 
 from command_runner import command_runner
 
-logger = logging.getLogger(__name__)
+logger = logging.getLogger(__intname__)
 FILE_LOCK = None
 
 
@@ -59,7 +88,11 @@ def _file_lock():
         FILE_LOCK.release()
 
 
-def check_path_access(path: str, check: str = "R") -> bool:
+def check_path_access(
+    path,  # type: str
+    check="R",  # type: str
+):
+    # type: (...) -> bool
     """
     Check if a path is accessible, if not, decompose path until we know which part isn't writable / readable
     when writable checks fail, we automatically fallback to readable tests
@@ -80,7 +113,10 @@ def check_path_access(path: str, check: str = "R") -> bool:
 
     logger.debug('Checking access to path "{0}"'.format(path))
 
-    def _check_path_access(sub_path: str):
+    def _check_path_access(
+        sub_path,  # type: str
+    ):
+        # type: (...) -> bool
         if os.path.exists(sub_path):
             obj = "file" if os.path.isfile(sub_path) else "directory"
             if obj == "file":
@@ -106,7 +142,7 @@ def check_path_access(path: str, check: str = "R") -> bool:
                         test_file = (
                             sub_path
                             + "/.somehopefullyunexistenttestfile"
-                            + str(datetime.now().timestamp())
+                            + str(get_timestamp(datetime.now()))
                         )
                         open(test_file, "w")
                         remove_file(test_file)
@@ -138,7 +174,10 @@ def check_path_access(path: str, check: str = "R") -> bool:
             )
         return False
 
-    def _split_path(path):
+    def _split_path(
+        path,  # type: str
+    ):
+        # type: (...) -> bool
         split_path = (path, "")
         can_split = True
         failed_once = False
@@ -164,21 +203,30 @@ def check_path_access(path: str, check: str = "R") -> bool:
     return not result
 
 
-def make_path(path: str):
+def make_path(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
         if not os.path.isdir(path):
             os.makedirs(path)
 
 
-def remove_file(path: str):
+def remove_file(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
         if os.path.isfile(path):
             os.remove(path)
 
 
-def remove_dir(path: str):
+def remove_dir(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
 
@@ -189,14 +237,25 @@ def remove_dir(path: str):
             shutil.rmtree(path)
 
 
-def move_file(source: str, dest: str):
+def move_file(
+    source,  # type: str
+    dest,  # type: str
+):
+    # type: (...) -> None
     make_path(os.path.dirname(dest))
     with _file_lock():
         # Using copy function because we don't want metadata, permissions, buffer nor anything else
-        shutil.move(source, dest, copy_function=shutil.copy)
+        if sys.version_info[0] >= 3:
+            shutil.move(source, dest, copy_function=shutil.copy)
+        else:
+            shutil.move(source, dest)
 
 
-def glob_path_match(path: str, pattern_list: list) -> bool:
+def glob_path_match(
+    path,  # type: str
+    pattern_list,  # type: list
+):
+    # type: (...) -> bool
     """
     Checks if path is in a list of glob style wildcard paths
     :param path: path of file / directory
@@ -206,7 +265,10 @@ def glob_path_match(path: str, pattern_list: list) -> bool:
     return any(fnmatch(path, pattern) for pattern in pattern_list)
 
 
-def log_perm_error(path: str) -> None:
+def log_perm_error(
+    path,  # type: str
+):
+    # type: (...) -> None
     """
     Default function that gets executed on get_paths_recursive permission error
     """
@@ -216,20 +278,21 @@ def log_perm_error(path: str) -> None:
 
 
 def get_paths_recursive(
-    root: str,
-    d_exclude_list: list = None,
-    f_exclude_list: list = None,
-    d_include_list: list = None,
-    f_include_list: list = None,
-    exclude_dirs: bool = False,
-    exclude_files: bool = False,
-    ext_exclude_list: list = None,
-    ext_include_list: list = None,
-    min_depth: int = 1,
-    max_depth: int = 0,
-    primary_root: str = None,
-    fn_on_perm_error: Callable = None,
-) -> Union[Iterable, str]:
+    root,  # type: str
+    d_exclude_list=None,  # type: list
+    f_exclude_list=None,  # type: list
+    d_include_list=None,  # type: list
+    f_include_list=None,  # type: list
+    exclude_dirs=False,  # type: bool
+    exclude_files=False,  # type: bool
+    ext_exclude_list=None,  # type: list
+    ext_include_list=None,  # type: list
+    min_depth=1,  # type: int
+    max_depth=0,  # type: int
+    primary_root=None,  # type: str
+    fn_on_perm_error=None,  # type: Callable
+):
+    # type: (...) -> Union[Iterable, str]
     """
     Walk a path to recursively find files
     Accepts glob style windcards for every list parameter except file extension lists
@@ -292,7 +355,10 @@ def get_paths_recursive(
         # Let's also make sure that min_depth parameter is used as in gnu find
         min_depth = min_depth - 1
 
-    def _find_files(min_depth):
+    def _find_files(
+        min_depth,  # type: int
+    ):
+        # type: (...) -> Iterable
         if min_depth < 1:
             try:
                 if not exclude_dirs:
@@ -319,7 +385,11 @@ def get_paths_recursive(
             except PermissionError:
                 pass
 
-    def _find_files_in_dirs(min_depth, max_depth):
+    def _find_files_in_dirs(
+        min_depth,  # type: int
+        max_depth,  # type: int
+    ):
+        # type: (...) -> Iterable
         min_depth = min_depth - 1
         if max_depth == 0 or max_depth > 1:
             max_depth = max_depth - 1 if max_depth > 1 else 0
@@ -367,16 +437,17 @@ def get_paths_recursive(
 
 
 def get_files_recursive(
-    root: str,
-    d_exclude_list: list = None,
-    f_exclude_list: list = None,
-    ext_exclude_list: list = None,
-    ext_include_list: list = None,
-    depth: int = 0,
-    primary_root: str = None,
-    fn_on_perm_error: Callable = None,
-    include_dirs: bool = False,
-) -> Union[Iterable, str]:
+    root,  # type: str
+    d_exclude_list=None,  # type: list
+    f_exclude_list=None,  # type: list
+    ext_exclude_list=None,  # type: list
+    ext_include_list=None,  # type: list
+    depth=0,  # type: int
+    primary_root=None,  # type: str
+    fn_on_perm_error=None,  # type: Callable
+    include_dirs=False,  # type: bool
+):
+    # type: (...) -> Union[Iterable, str]
     """
     Wrapper for ofunctions.file_utils < 0.9.0 code
     """
@@ -396,12 +467,13 @@ def get_files_recursive(
 
 
 def replace_in_file(
-    source_file: str,
-    text_to_search: str,
-    replacement_text: str,
-    dest_file: str = None,
-    backup_ext: str = None,
-) -> None:
+    source_file,  # type: str
+    text_to_search,  # type: str
+    replacement_text,  # type: str
+    dest_file=None,  # type: str
+    backup_ext=None,  # type: str
+):
+    # type: (...) -> None
     """
 
     :param source_file: source file to replace text in
@@ -430,7 +502,11 @@ def replace_in_file(
         fp.write(data)
 
 
-def get_file_time(path_to_file: str, mac_type: str = "ctime") -> float:
+def get_file_time(
+    path_to_file,  # type: str
+    mac_type="ctime",  # type: str
+):
+    # type: (...) -> float
     """
     Returns file ctime/mtime/atime
 
@@ -458,14 +534,20 @@ def get_file_time(path_to_file: str, mac_type: str = "ctime") -> float:
             return stat.st_mtime
 
 
-def file_creation_date(path_to_file: str) -> float:  # COMPAT <0.7.8
+def file_creation_date(
+    path_to_file,  # type: str
+):  # COMPAT <0.7.8
+    # type: (...) -> float
     """
     Wrapper for get_file_time (kept for compatibility reasons)
     """
     return get_file_time(path_to_file, "ctime")
 
 
-def file_modification_date(path_to_file: str) -> float:  # COMPAT <0.7.8
+def file_modification_date(
+    path_to_file,  # type: str
+):  # COMPAT <0.7.8
+    # type: (...) -> float
     """
     Wrapper for get_file_time (kept for compatibility reasons)
     """
@@ -473,45 +555,52 @@ def file_modification_date(path_to_file: str) -> float:  # COMPAT <0.7.8
 
 
 def check_file_timestamp_delta(
-    file: str,
-    mac_type: str = "ctime",
-    years: int = 0,
-    days: int = 0,
-    hours: int = 0,
-    minutes: int = 0,
-    seconds: int = 0,
-) -> bool:
+    file,  # type: str
+    mac_type="ctime",  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+    timestamp=None,  # type: float
+):
+    # type: (...) -> bool
     """
     mac_type can be ctime, mtime, or atime for comparison purposes
 
-    Simple check if a file is newer/older (ctime) than given time delta from now
-    Can also check if file has been modified (mtime) earlier than given time delta from now
-
-    Future timestamps are achieved by specifying positive values, eg days=1
-    Past timestamps are achieved by specifying negative values, eg days=-1
+    Simple check if a file is newer/older (ctime) than given time delta from timestamp
+    Can also check if file has been modified (mtime) earlier than given time delta from timestamp
+    If no timestamp is given, we'll use current time
 
+    future comparaisons are achieved by specifying postivie values, ex: days=1 would search for files created / modified / accessed tomorrow
+    past comparaisons are achieved by specifying negative values, ex: days=-1 would search for files created / modified / accessed yesterday
     """
     if not os.path.isfile(file):
         raise FileNotFoundError("[%s] not found." % file)
     delta = (
         seconds + (minutes * 60) + (hours * 3600) + (days * 86400) + (years * 31536000)
     )
-    # file creation date is UTC for Linux, TZ for Windows
-    if os.name == "nt":
-        now = datetime.now().timestamp()
+
+    if not timestamp:
+        if os.name == "nt" or sys.version_info[0] < 3:
+            # file creation date is UTC for Linux Python 3+, TZ for Windows or Linux Python 2.7
+            now = get_timestamp(datetime.now())
+        else:
+            now = get_timestamp(datetime.utcnow())
     else:
-        now = datetime.utcnow().timestamp()
+        now = timestamp
     return bool((now + delta - get_file_time(file, mac_type)) > 0)
 
 
 def is_file_older_than(
-    file: str,
-    years: int = 0,
-    days: int = 0,
-    hours: int = 0,
-    minutes: int = 0,
-    seconds: int = 0,
-) -> bool:  # COMPAT <0.7.8
+    file,  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):  # COMPAT <0.7.8
+    # type: (...) -> bool
     """
     Wrapper for check_file_times kept for compatibility
     """
@@ -528,49 +617,52 @@ def is_file_older_than(
 
 
 def remove_files_on_timestamp_delta(
-    directory: str,
-    mac_type: str = "ctime",
-    years: int = 0,
-    days: int = 0,
-    hours: int = 0,
-    minutes: int = 0,
-    seconds: int = 0,
-) -> None:
+    directory,  # type: str
+    mac_type="ctime",  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+    timestamp=None,  # type: Optional[datetime]
+):
+    # type: (...) -> None
     """
-    Remove files older than given delta from now
+    Remove files older than given delta from timestamp
+    If no timestamp is given, we'll use current date timestamp
     """
 
     if not os.path.isdir(directory):
         raise FileNotFoundError("[%s] not found." % directory)
 
-    for _, _, filenames in os.walk(directory):
-        for filename in filenames:
-            filename = os.path.join(directory, filename)
-            try:
-                if check_file_timestamp_delta(
-                    filename,
-                    mac_type=mac_type,
-                    years=years,
-                    days=days,
-                    hours=hours,
-                    minutes=minutes,
-                    seconds=seconds,
-                ):
-                    os.remove(filename)
-            except FileNotFoundError:
-                pass
-            except (IOError, OSError):
-                raise OSError("Cannot remove file [%s]." % filename)
+    for filename in get_paths_recursive(directory, exclude_dirs=True):
+        try:
+            if check_file_timestamp_delta(
+                filename,
+                mac_type=mac_type,
+                years=years,
+                days=days,
+                hours=hours,
+                minutes=minutes,
+                seconds=seconds,
+                timestamp=timestamp,
+            ):
+                os.remove(filename)
+        except FileNotFoundError:
+            pass
+        except (IOError, OSError):
+            raise OSError("Cannot remove file [%s]." % filename)
 
 
 def remove_files_older_than(
-    directory: str,
-    years: int = 0,
-    days: int = 0,
-    hours: int = 0,
-    minutes: int = 0,
-    seconds: int = 0,
-) -> None:  # COMPAT <0.7.8
+    directory,  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):  # COMPAT <0.7.8
+    # type: (...) -> None
     """
     Wrapper for remove_files_on_timestamp_delta for compatibility
     """
@@ -586,31 +678,49 @@ def remove_files_older_than(
     )
 
 
-def remove_bom(file: str) -> None:
+def remove_bom(
+    file,  # type: str
+):
+    # type: (...) -> None
     """
     Remove BOM from existing UTF-8 file
     We don't use any utf-8-sig codec magic here to avoid any UnicodeDecodeErrors
     hence the function is uglier than it should, but less error prone
     """
+
+    buffer = 32768
+
     try:
         with open(file, "rb") as file_handle_in:
             data = file_handle_in.read(3)
             # Throw away the data if it's a BOM
             if data == b"\xef\xbb\xbf":
-                data = file_handle_in.read(32768)
+                data = file_handle_in.read(buffer)
             else:
                 return
             with open(file + ".tmp", "wb") as file_handle_out:
                 while len(data) > 0:
                     file_handle_out.write(data)
-                    data = file_handle_in.read(32768)
+                    data = file_handle_in.read(buffer)
         if os.path.isfile(file + ".tmp"):
-            os.replace(file + ".tmp", file)
-    except Exception:
-        raise OSError
+            # Python 2.7 does not have os.replace()
+            try:
+                os.replace(file + ".tmp", file)
+            except AttributeError:  # Python 2 workaround
+                try:
+                    os.remove(file)
+                except:
+                    pass
+                os.rename(file + ".tmp", file)
+    except Exception as exc:
+        raise OSError(exc)
 
 
-def write_json_to_file(file: str, data: Union[dict, list]):
+def write_json_to_file(
+    file,  # type: str
+    data,  # type: Union[dict, list]
+):
+    # type: (...) -> None
     """
     Creates a manifest to the file containing it's sha256sum and the installation result
 
@@ -623,7 +733,10 @@ def write_json_to_file(file: str, data: Union[dict, list]):
         json.dump(data, file_handle, ensure_ascii=False)
 
 
-def read_json_from_file(file: str) -> dict:
+def read_json_from_file(
+    file,  # type: str
+):
+    # type: (...) -> dict
     """
     Verifies exit code of the file manifest and returns True if installed successfully
     :param file: (str) path to file
@@ -638,25 +751,52 @@ def read_json_from_file(file: str) -> dict:
         return {}
 
 
-def grep(file: str, pattern: str) -> list:
+def grep(
+    file,  # type: str
+    pattern,  # type: str
+    ignorecase=False,  # type bool
+):
+    # type: (...) -> list
+    """
+    Grep emulation
+    """
     if not os.path.isfile(file):
         raise FileNotFoundError(file)
     result = []
+
     with open(file, "r") as file_handle:
         for line in file_handle:
-            if re.search(pattern, line):
-                result.append(line)
+            if ignorecase:
+                if re.search(pattern, line, re.IGNORECASE):
+                    result.append(line)
+            else:
+                if re.search(pattern, line):
+                    result.append(line)
     return result
 
 
-def hide_windows_file(file: str, hidden: bool = True) -> bool:
+def hide_windows_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides / unhides a file under windows by using attrib command
+    """
     result, _ = command_runner('attrib %sh "%s"' % ("+" if hidden else "-", file))
     if result == 0:
         return True
     return False
 
 
-def hide_unix_file(file: str, hidden: bool = True) -> bool:
+def hide_unix_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides / unhides a file under unix by prepending a dot
+    """
     if (file.startswith(".") and hidden) or (not file.startswith(".") and not hidden):
         return True
 
@@ -673,14 +813,22 @@ def hide_unix_file(file: str, hidden: bool = True) -> bool:
         return False
 
 
-def hide_file(file: str, hidden: bool = True) -> bool:
+def hide_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides/unhindes a file under Windows / Unix platforms
+    """
     if os.name == "nt":
         return hide_windows_file(file, hidden)
     else:
         return hide_unix_file(file, hidden)
 
 
-def get_writable_temp_dir() -> Optional[str]:
+def get_writable_temp_dir():
+    # type: (...) -> Optional[str]
     """
     Try to find a writable temporary directory
     """
@@ -702,7 +850,10 @@ def get_writable_temp_dir() -> Optional[str]:
     return None
 
 
-def get_writable_random_file(ident_str: str = "tmp_file_utils") -> Optional[str]:
+def get_writable_random_file(
+    ident_str="tmp_file_utils",  # type: str
+):
+    # type: (...) -> Optional[str]
     """
     Try to return a path to a not yet existing random file
     """
